@@ -178,7 +178,9 @@ print.bn = function(x, ...) {
   # warn about unused arguments.
   check.unused.args(list(...), character(0))
 
-  if (x$learning$test %in% names(test.labels))
+  if (x$learning$test == "none")
+    cat("\n  Randomly generated Bayesian network\n\n")
+  else if (x$learning$test %in% names(test.labels))
     cat("\n  Bayesian network learned via Conditional Independence methods\n\n")
   else
     cat("\n  Bayesian network learned via Scoring methods\n\n")
@@ -196,9 +198,10 @@ print.bn = function(x, ...) {
 
   cat("\n")
 
-  cat("  learning algorithm:                   ", method.labels[x$learning$algo], "\n")
-  if (x$learning$algo != "rnd") {
+  if (x$learning$test != "none") {
 
+    cat("  learning algorithm:                   ", method.labels[x$learning$algo], "\n")
+  
     if (x$learning$test %in% names(test.labels))
       cat("  conditional independence test:        ", test.labels[x$learning$test], "\n")
     else
@@ -216,6 +219,22 @@ print.bn = function(x, ...) {
     cat("  tests used in the learning procedure: ", x$learning$ntests, "\n")
 
   }#THEN
+  else {
+
+    cat("  generation algorithm:                 ", graph.generation.labels[x$learning$algo], "\n")
+
+    if ("prob" %in% params)
+      cat("  arc sampling probability:             ", x$learning$args$prob, "\n")
+    if ("burn.in" %in% params)
+      cat("  burn in length:                       ", x$learning$args$burn.in, "\n")
+    if ("max.in.degree" %in% params)
+      cat("  maximum in-degree:                    ", x$learning$args$max.in.degree, "\n")
+    if ("max.out.degree" %in% params)
+      cat("  maximum out-degree:                   ", x$learning$args$max.out.degree, "\n")
+    if ("max.degree" %in% params)
+      cat("  maximum degree:                       ", x$learning$args$max.degree, "\n")
+
+  }#ELSE
 
   cat("\n")
 
